%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           FUNCION FILTRO_PELO     
%   Realiza una estimacion de del componente de vello en una imagen de la
%   piel. Generara una imagen binaria que posteriormente se utilizara para
%   desechar las marcas sobre el pelo. Como entradas necesitara la imagen
%   original, el tamao deseado para el filtro y a que nivel de gris se va
%   a establecer el umbral.%   
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function y= filtro_pelo(imagen,tamano,umbral);

k=tamano^2-1; % Se calcula el numero de pixels que va a tener el filtro (sin tener en cuenta el central)
tamano=floor(tamano/2); % Se calcula cuantos pixels vecinos respecto del central hay que sumar o restar para el calculo del filtro  

imagenbn=blanconegro(imagen); % se realiza una copia de la imagen en blanco y negro
[M,N]=size(imagenbn); % se realiza una copia de la imagen en blanco y negro
imagen=double(imagen); % se transforman las imagenes a double para poder realizar operacines matematicas
imagenbn=double(imagenbn);

for i=1:M 
    for j=1:N % bucle anidado que recorer cada pixel de la imagen para calcular el filtro para cada uno de ellos
        suma=0; % se restablece la suma a 0
        for a=-tamano:tamano 
            for b=-tamano:tamano % bucles anidados que recorren los pixels que entraran en la mascara del filtro
                coord1=i+a; 
                coord2=j+b; % Las coordenadas de cada pixel de la mascara se calculan mediante sumar las coordenadas parciales del pixel dentro de la mascara a las coordenadas globales del pixel central
                if a==0 & b==0 % si a y b son 0, se trata del pixel central, que en la mascara adquiere el valor de la suma del total de pixels en la mascara menos 1
                    suma=suma+k*imagenbn(coord1,coord2); % se aade a la suma el valor para el pixel central
                elseif coord1>0 & coord1<M & coord2>0 & coord2<N % si la mascara no se sale del rango de la imagen se resta el valor de pixel a la suma
                    suma=suma-imagenbn(coord1,coord2); 
                else % En el resto de casos la mascara se sale del rango de la imagen
                    suma=0; % se le asigna a la suma el valor 0
                end
                y(i,j)=suma; % se guarda el valor de la suma para el pixel correspondiente en la variable y
            end
        end
    end
end
y=uint8(y); % se transforma a uint8 porque no se van a realizar mas operaciones matematicas



for i=1:M 
    for j=1:N % se recorre la imagen pixel a pixel
        if y(i,j)>umbral % si el pixel tiene un valor superior se le asignara el valor 255
            y(i,j)=255;
        else % si no supera el umbral se le asigna el valor 0, generando una imagen binaria
            y(i,j)=0;
        end
    end
end

 y=dilatacion(y,5); % se realiza la dilatacion para incluir marcas finas como pelo, manteniendo las gruesas
 
 
 %tendremos una primera estimacion de pelo, pero para obtener mejores
 %reswultados, tendremos en cuenta el grosor de las marcas generadas, ya
 %que en el caso de que haya pelo, se generara una "masa" cuya anchura sera
 %superior a la de las pecas.
 
 
 for i=1:M
    for j=1:N %se recorre cada pixel de la imagen
        if y(i,j)==255 % si se trata de un pixel blanco pasaremos a realizar las siguientes operaciones
           suma=0; % se inicializa la suma a 0
           for a=j:N  % se van comprobando si los pixels de las columnas siguientes son blancos
            if y(i,a)==255 
                suma=suma+1; %por cada pixel blanco se suma uno a la variable
            else
                break % cuando encontremos un pixel negro saltaremos del bucle
            end
           end
           if suma<40 % comprobaremos si el grosor de la marca es inferior a 40, puede tratarse de una peca, asi que no consideraremos estos pixels como pelo
                y(i,j:a)=0;
           end
        end
    end
end

% se podria realizar el barrido desde otras direcciones, pero el resultado
% apenas mejora (ademas de complicar y ralentizar el procesado), por ello se opto por no realizarlo.