%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           FUNCION PIEL                       
%  Realiza una estimacion de lo que es el fondo y y la superficie
%  correspondiente a la piel en la imagen. Generara una imagen binaria
%  donde los pixels blancos corresponderan al fondo y los negros a la
%  piel. Despues se utilizara esta imagen para eliminar marcas que se
%  encuentren en la zona correspondiente al fondo. Como entrada necesitara
%  la imagen marcada.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function y=piel(imagen);

%primero calculara las dimensiones de la imagen, la transforma a double y
%crea unaq copia en el dominio HSV para luego hacer las comparaciones.
[M,N]=size(imagen);
N=N/3;
ima=double(imagen);
imahsv=rgb2hsv(ima);



for i=1:M %bucles anidados que recorren la imagen
     for j=1:N        
         if imagen(i,j,2)>250 %se comprueba si corresponde a un pixel verde
            y(i,j)=0; % en el caso de que si sea, se estima como piel 
            break %salta del bucle, con esto conseguimos agilizar la velocidad de ejecucion del programa
         end        
        r=ima(i,j,1); % si el pixel no es verde, se extraen sus caracteristicas de color
        g=ima(i,j,2);
        b=ima(i,j,3);
        gris=round((r+g+b)/3);
        s=imahsv(i,j,2);
        v=imahsv(i,j,3);   
        if r>g & r>b & g+6>b & r>70 & g>40 & g<250 & b>25 & b<230 & gris>50 & gris<245 & s<75 & v>25 % se comprueba si el pixel cumple las caracteristicas de color
          y(i,j)=0;% en el caso de que si, se considerara como piel
        else
          y(i,j)=255; % en el caso de que no, se considerara como fondo
        end
    end
end

% repetiremos la operacion empezando desde las otras tres esquinas para
% conseguir un resultado optimo
for i=1:M
    for j=N:-1:1        
        if imagen(i,j,2)>250
             y(i,j)=0;
            break
        end        
        r=ima(i,j,1);
        g=ima(i,j,2);
        b=ima(i,j,3);
        gris=round((r+g+b)/3);
        s=imahsv(i,j,2);
        v=imahsv(i,j,3);   
        if not(r>g & r>b & g+6>b & r>70 & g>40 & g<250 & b>25 & b<230 & gris>50 & gris<245 & s<75 & v>25 ) 
          y(i,j)=255;
      end
    end
end

for j=1:N
    for i=1:M        
        if imagen(i,j,2)>250
             y(i,j)=0;
            break
        end        
        r=ima(i,j,1);
        g=ima(i,j,2);
        b=ima(i,j,3);
        gris=round((r+g+b)/3);
        s=imahsv(i,j,2);
        v=imahsv(i,j,3);   
       if not(r>g & r>b & g+6>b & r>70 & g>40 & g<250 & b>25 & b<230 & gris>50 & gris<245 & s<75 & v>25 ) 
          y(i,j)=255;
      end
    end
end

for j=1:N
    for i=M:-1:1
        if imagen(i,j,2)>250
             y(i,j)=0;
          break
        end        
        r=ima(i,j,1);
        g=ima(i,j,2);
        b=ima(i,j,3);
        gris=round((r+g+b)/3);
        s=imahsv(i,j,2);
        v=imahsv(i,j,3);   
       if not(r>g & r>b & g+6>b & r>70 & g>40 & g<250 & b>25 & b<230 & gris>50 & gris<245 & s<75 & v>25 ) 
          y(i,j)=255;
        end
    end
end

%realizaremos una dilatacion para considerar como fondo los pixels
%correspondientes al borde entre el fondo i la piel de la imagen
y=dilatacion(y,10);
%tendremos una primera estimacion de lo que es la piel




z=zeros(M,N);% se genera una imagen totalmente negra con las dimensiones de la original

%los siguientes dos bucles marcan como fondo los pixels correspondientes a
%los limites de la imagen
for i=1:N
    z(1,i)=255;    
    z(M,i)=255;
end
for i=1:M
    z(i,1)=255;
    z(i,N)=255;
end


%recorreremos la imagen considerando como fondo los pixels que tengan como
%vecino al menos uno blanco y que tambien sean blancos en la primera
%estimacion.Con esta tecnica habremos eliminado de la primera estimacion
%partes que se encuentran rodeadas de piel como puede ser el pelo
for i=2:M-1
    for j=2:N-1        
          if (z(i-1,j)==255 | z(i+1,j)==255 | z(i,j-1)==255 | z(i,j+1)==255)& y(i,j)==255 % comprobamos si algun vecino es blanco y si en la primera estimacion esta incluido
                z(i,j)=255;% si se cumple consideraremos el pixel como fondo
            end
    end
end
z1=z;
% guardamos el resultado en Z1 y repetimos la operacion empezando por las
% otras tres esquinas, generando Z2, Z3 y Z4

z=zeros(N,M);
for i=1:N
    z(i,1)=255;
    z(i,M)=255;
end
for i=1:M
    z(1,i)=255;
    z(N,i)=255;
end

for i=1:M
    for j=1:N        
         y2(j,i)=y(i,j);
    end
end
for i=2:N-1
    for j=2:M-1        
          if (z(i-1,j)==255 | z(i+1,j)==255 | z(i,j-1)==255 | z(i,j+1)==255)& y2(i,j)==255
                z(i,j)=255;
            end
    end
end
z2=z;




z=zeros(M,N);
for i=1:M
    z(i,1)=255;
    z(i,N)=255;
end
for i=1:N
    z(1,i)=255;
    z(M,i)=255;
end
for i=1:M-1
    for j=1:N-1        
         y3(M-i,N-j)=y(i,j);
    end
end
for i=2:M-1
    for j=2:N-1        
          if (z(i-1,j)==255 | z(i+1,j)==255 | z(i,j-1)==255 | z(i,j+1)==255)& y3(i,j)==255
                z(i,j)=255;
            end
    end
end
z3=z;







z=zeros(N,M);
for i=1:N
    z(i,1)=255;
    z(i,M)=255;
end
for i=1:M
    z(1,i)=255;
    z(N,i)=255;
end
for i=1:M-1
    for j=1:N-1        
         y3(N-j,M-i)=y(i,j);
    end
end
for i=2:N-1
    for j=2:M-1        
          if (z(i-1,j)==255 | z(i+1,j)==255 | z(i,j-1)==255 | z(i,j+1)==255)& y3(i,j)==255
                z(i,j)=255;
            end
    end
end
z4=z;




%Aadiremos los pixels considerados como fon do en las variables z1, z2, z3
%y z4 a la primera estimacion
for i=1:M 
    for j=1:N
        if z1(i,j)==255 | z2(j,i)==255 | z3(M+1-i,N+1-j)==255 | z4(N-j+1,M-i+1)==255
            y(i,j)=255;
        else
            y(i,j)=0;
        end
    end
end

%Realizaremos una dilatacion para eliminar las marcas sobre los bordes de
%piel y fondo, obteniendo el resultado final.
y=dilatacion(y,11); 
