%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           FUNCION PREWITT2                       
%  Realiza un filtrado Prewitt vertical sobre la imagen en escala 
%  de grises para obtener otra imagen con los pixels correspondientes 
%  a los bordes verticales remarcados. Como entrada necesita la imagen en
%  escala de grises y como salida dara otra imagen en escala de grises
%  lista para umbralizar y marcar sobre la original
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function T=prewitt2(f)

f=double(f); % transforma imagen a double para poder realizar operaciones matematicas
[M,N]=size(f); % calcula las dimensiones de la imagen

for i=1:M % doble bucle que va recorriendo cada pixel de la imagen
    for j=1:N
        
         % Se ira comprobando si los pixels corresponden a bordes u esquinas
        % de la imagen para ver cuantos vecinos habra que tener en cuenta.
        % Despues se multiplicara cada vecino por su coeficiente
        % correspondiente y se realizara la suma, obteniendo el valor del
        % filtro para ese pixel en concreto
        
        if i==1
            if j==1 %comprueba si el pixel pertenece a la esquina superior izquierda
                T(i,j)=f(i,j+1)+f(i+1,j+1);
            elseif j==N %comprueba si el pixel pertenece a la esquina superior derecha
                T(i,j)=-f(i,j-1)-f(i+1,j-1);
            else %comprueba si el pixel pertenece a la  fila de arriba
                T(i,j)=-f(i,j-1)-f(i+1,j-1)+f(i,j+1)+f(i+1,j+1);
            end
        elseif i==M
            if j==1 %comprueba si el pixel pertenece a la esquina inferior izquierda
                T(i,j)=f(i-1,j+1)+f(i,j+1);
            elseif j==N %comprueba si el pixel pertenece a la esquina inferior derecha
               T(i,j)=-f(i-1,j-1)-f(i,j-1);
            else %comprueba si el pixel pertenece a la  fila de abajo
                T(i,j)=-f(i-1,j-1)-f(i,j-1)+f(i-1,j+1)+f(i,j+1);
            end
        else
            if j==1 %comprueba si el pixel pertenece a la columna de la izquierda
               T(i,j)=f(i-1,j+1)+f(i,j+1)+f(i+1,j+1);
            elseif j==N %comprueba si el pixel pertenece a la  columna de la derecha
                T(i,j)=-f(i-1,j-1)-f(i,j-1)-f(i+1,j-1);
            else %pertenece a cualquier otra parte de la imagen
                T(i,j)=-f(i-1,j-1)-f(i,j-1)-f(i+1,j-1)+f(i-1,j+1)+f(i,j+1)+f(i+1,j+1);
            end
        end
    end
end

T=escala256(T); % Tras los calculos el rengo dinamico ha cambiado, se vuelve a meter en el rango 0-256
T=uint8(T); % se transforma a uint8 para dar el resultado final

       