%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           FUNCION PROCESADO2     
%   Corrige marcas erroneas que se han obtenido tras un primer procesado.
%   Se ayudara de caracteristicas como color y forma de las marcas para
%   poder deshecharlas. El algoritmo llamara a otras subfunciones que se
%   encargaran de trabajos mas especificos (eliminacion del fondo, filtros para el pelo,
%   bordes,etc...) Como variables de entrada precisara de la imagen
%   original y de la imagen marcada resultado del primer procesado. Dara
%   tres resultados diferentes: uno sin aplicar filtros, otro aplicando un
%   filtro para el pelo y otro aplicando un filtro para los bordes. El
%   resultado optimo dependera de las caracteristicas de la imagen. Los
%   tres resultados seran guardadados en tres imagenes .bmp diferentes.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function y= procesado2(marcada,original)

orig=original; % carga la imagen original y calcula sus dimensiones
[M,N]=size(orig);
N=N/3;
y=marcada; % Realiza una copia de la imagen marcada y la transforma para poder realizar operacines matematicas.
y=double(y);

%Los siguientes bucles anidados, recorren los bordes de la imagen para borrar las
%marcas que han producido. En un marco de 6 pixels de grosor de la imagen,
%se asiganara a la imagen marcado los valores de la original.
for i=1:6
    for j=1:N
        y(i,j,1)=orig(i,j,1);
        y(i,j,2)=orig(i,j,2);    
        y(i,j,3)=orig(i,j,3);    
    end
end
for i=M-6:M
    for j=1:N
        y(i,j,1)=orig(i,j,1);
        y(i,j,2)=orig(i,j,2);    
        y(i,j,3)=orig(i,j,3);    
    end
end
for i=1:M
    for j=1:6
        y(i,j,1)=orig(i,j,1);
        y(i,j,2)=orig(i,j,2);    
        y(i,j,3)=orig(i,j,3);    
    end
end
for i=1:M
    for j=N-6:N
        y(i,j,1)=orig(i,j,1);
        y(i,j,2)=orig(i,j,2);    
        y(i,j,3)=orig(i,j,3);    
    end
end

y=borra(y,orig); % Funcion que elimina marcas sobre zonas que no cumplen los requisitos de color

fondo=piel(y);% Genera una imagen binaria diferenciando fondo y piel
for i=1:M %bucle que borra las marcas localizadas en el fondo
    for j=1:N
        if fondo(i,j)==255
           y(i,j,1)=orig(i,j,1);
           y(i,j,2)=orig(i,j,2);    
           y(i,j,3)=orig(i,j,3);    
        end
    end
end

filtro=filtro_pelo(orig,5,150); % Funcion que hace una estimacion de las zonas de pelo
y2=eliminar_pelo(y,orig,filtro); % Aplica el filtro para el pelo

y=Extraccion_contornos(y);% realiza la extraccion de contornos mediante operadores morfologicos
 
orig=imread('original.bmp'); % Se marca sobre la imagen original y se guarda el primer de los resultados (sin aplicar filtros)
for i=1:M
    for j=1:N
        if y(i,j)==255 
           y1(i,j,1)=0;
           y1(i,j,2)=255;
           y1(i,j,3)=0;
       else
           y1(i,j,1)=orig(i,j,1);
           y1(i,j,2)=orig(i,j,2);
           y1(i,j,3)=orig(i,j,3);
        end
    end
end
y1=uint8(y1);
imwrite(y1,'Resultado_sin_filtros.bmp');
y1=double(y1);

y=estudio_contorno(y);% Elimina las marcas con formas que no pertenecen a pecas

orig=imread('original.bmp'); % Se marca sobre la imagen original y se guarda el resultado para el filtro de bordes
for i=1:M
    for j=1:N
        if y(i,j)==255 
           y1(i,j,1)=0;
           y1(i,j,2)=255;
           y1(i,j,3)=0;
       else
           y1(i,j,1)=orig(i,j,1);
           y1(i,j,2)=orig(i,j,2);
           y1(i,j,3)=orig(i,j,3);
        end
    end
end
y1=uint8(y1);
imwrite(y1,'Resultado_filtro_bordes.bmp');
y1=double(y1);

y2=Extraccion_contornos(y2);% se realiza la extraccion para el filtro de pelo

for i=1:M % Se marca sobre la imagen original y se guarda el resultado para el filtro de pelo
    for j=1:N
        if y2(i,j)==255
           y1(i,j,1)=0;
           y1(i,j,2)=255;
           y1(i,j,3)=0;
       else
           y1(i,j,1)=orig(i,j,1);
           y1(i,j,2)=orig(i,j,2);
           y1(i,j,3)=orig(i,j,3);
        end
    end
end
y1=uint8(y1);
imwrite(y1,'Resultado_filtro_pelo.bmp');


